package jpatp.column;

import java.util.List;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityManager;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.EntityTransaction;

public class TestMain {
    private EntityManagerFactory factory;
    private EntityManager manager;

    public TestMain() {

        // get EntityManager
        factory = Persistence.createEntityManagerFactory("em1");
        manager = factory.createEntityManager();
        

        // get data from DB and initialize the list
        Query query = manager.createNativeQuery("SELECT * FROM PERSONENTITY p ORDER BY p.id", PersonEntity.class);
        // Compilation warning: evade no inspection Cast
        List<PersonEntity> results = PersonEntities.<PersonEntity>getList(query.getResultList());


        // get transaction
        EntityTransaction transaction = manager.getTransaction();

        try{
            // transaction starts
            transaction.begin();

            // data updates
            for(PersonEntity person : results) {
                System.out.println("### id = " + person.getId() + " FirstName = " + person.getFirstName() + " LastName = " + person.getLastName2());
                // change LastName2 of the Entity to hoge
                person.setLastName2("hoge");
                manager.merge(person);
                System.out.println("### id = " + person.getId() + " FirstName = " + person.getFirstName() + " LastName = " + person.getLastName2());
            }

            // commit transaction
            transaction.commit();

        }
        catch(Throwable th){
            System.err.println("### Unexpected exception occured.");
            System.err.println("### " + th.getMessage());
            th.printStackTrace();
            if (transaction != null && transaction.isActive()){
                transaction.rollback();
            }
        }
        finally{
            // close EntityManager
            if (manager != null && manager.isOpen()){
                manager.close();
            }
            // close EntityManagerFactory
            if (factory != null && factory.isOpen()){
                factory.close();
            }
        }
    }

    public static void main(String[] args) {
        @SuppressWarnings("unused") TestMain testMain = new TestMain();
    }
}
